/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFReader;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorData;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorEventKey;
import nl.cloudfarming.client.sensor.SensorProcessor;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.NbBundle;

/**
 * GreenSeeker File processor abstract class. Each inheriting GreenSeeker file
 * processor implements its own parseLine method
 *
 * @author Arjan Loermans
 */
public abstract class GreenseekerFileProcessor extends SensorProcessor<GreenseekerSensorData> {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    private static final String LOGKEY_PROCESSING_PARTIALLY_PROCESSEDMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.partiallyprocessed");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_start");
    // Constant int values
    private static final int BULK_COMMIT_AMOUNT = 100; // How many record should be gathered before doing a bulk-commit?
    private static final int PROGRESS_REPORT_INTERVAL = 5000; // How many record should be processed before printing a progress message in the log
    // Other variable declarations
    private static final AppLogger LOG = AppLogFactory.getLogger(GreenseekerFileProcessor.class);
    private final ProgressHandle progressHandle;
    private final EventProducer eventProducer = new EventProducer();
    protected GreenseekerDataObject dataObject;

    /**
     * Create a GreenseekerFileProcessor instance to process the specified
     * GreenSeeker sensor file.
     *
     * @param toProcess The GreenSeeker sensor file to be processed
     */
    public GreenseekerFileProcessor(GreenseekerDataObject toProcess) {
        assert toProcess != null;
        this.dataObject = toProcess;
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(SensorProcessor.class, "sensor.common.fileprocessor.loginfo.process_description", dataObject.getName()));
    }

    abstract protected GreenseekerSensorData parseLine(Object[] payload) throws GreenseekerParseException;

    @Override
    public List<SensorData> process() {
        List<SensorData> sensorData = new ArrayList<SensorData>();
        try {
            int success = 0;
            int fail = 0;
            int progress = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), dataObject.getName());

            FileObject fileObject = dataObject.getPrimaryFile();

            // create a DBFReader object
            InputStream inputStream = new FileInputStream(FileUtil.toFile(fileObject));
            DBFReader reader = new DBFReader(inputStream);

            Object[] dbfSensorRow;
            // Start progressbar
            this.progressHandle.start(reader.getRecordCount());


            while ((dbfSensorRow = reader.nextRecord()) != null) {

                try {
                    sensorData.add(parseLine(dbfSensorRow));
                    success++;
                } catch (GreenseekerParseException ex) {
                    LOG.trace(ex);
                    fail++;
                }
                progressHandle.progress(++progress);
            }
            LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);
            if (fail > 0) {
                LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
            }
            eventProducer.triggerEvent(SensorEventKey.SENSOR_DATA_IMPORTED);
            progressHandle.finish();
            inputStream.close();
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, dataObject.getName());
            LOG.trace(ex);
        }
        return sensorData;
    }

    /**
     * Generic greenseeker method to build a Greenseeker specific metadata
     * object
     *
     * @return Greenseeker metadata object
     */
    @Override
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
        int sensorHeigth = Integer.parseInt(GreenseekerPreference.GREENSEEKER_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(GreenseekerPreference.NUMBER_OF_GREENSEEKER_SENSORS.getValue());
        String greenseekerSensorSpread = GreenseekerPreference.GREENSEEKER_SENSOR_SPREAD.getValue();
        String sensorType = GreenseekerPreference.SENSOR_TYPE.getValue();
        String spectralChannels = GreenseekerPreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, greenseekerSensorSpread, sensorType, spectralChannels);
    }

    @Override
    protected String getFileDateFormat() {
        return GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    private class EventProducer extends BaseGuiEventProducer {

        public EventProducer() {
            super(SensorEventKey.class);
        }
    }
}
